#!bin/bash

allkextsupper="ACPIBatteryManager AirportBrcmFixup AppleALC ATH9KFixup AtherosE2200Ethernet AzulPatcher4600 BrcmPatchRam BT4LEContiunityFixup CodecCommander CPUFriend EnableLidWake FakePCIID FakeSMC HibernationFixup IntelMausiEthernet Lilu LiluFriend NightShiftUnlocker NoTouchID NullCpuPowerManagement NullEthernet RealtekRTL8111 USBInjectAll VirtualSMC VoodooHDA VoodooI2C VoodooPS2 WhateverGreen"
allkextslower=`echo "$allkextsupper" | tr '[:upper:]' '[:lower:]'`

################################################################
####################### Helper Function ########################
################################################################

function _helpDefaultWrite()
{
    VAL=$1
    local VAL1=$2

    if [ ! -z "$VAL" ] || [ ! -z "$VAL1" ]; then
    defaults write "${ScriptHome}/Library/Preferences/com.slsoft.kextupdater.plist" "$VAL" "$VAL1"
    fi
}

function _helpDefaultRead()
{
    VAL=$1

    if [ ! -z "$VAL" ]; then
    defaults read "${ScriptHome}/Library/Preferences/com.slsoft.kextupdater.plist" "$VAL"
    fi
}

function _helpDefaultDelete()
{
    VAL=$1

    if [ ! -z "$VAL" ]; then
    defaults delete "${ScriptHome}/Library/Preferences/com.slsoft.kextupdater.plist" "$VAL"
    fi
}
################################################################
####################### Helper Function End ####################
################################################################

ScriptHome=$(echo $HOME)
ScriptTmpPath=`_helpDefaultRead "Temppath"`
ScriptReportPath=`_helpDefaultRead "Reportpath"`
MY_PATH="`dirname \"$0\"`"
cd "$MY_PATH"

#========================= Kext Array =========================#
## Script Name,kextstat Name, echo Name, Ersatz Name
#

kextArray=(
"acpibatterymanager","org.rehabman.driver.AppleSmartBatteryManager","ACPI BatteryManager",""
"airportbrcmfixup","BrcmWLFixup","BrcmWLFixup","AirportBrcmFixup"
"airportbrcmfixup","AirportBrcmFixup","AirportBrcmFixup",""
"applealc","AppleALC","AppleALC",""
"ath9kfixup","ATH9KFixup","ATH9KFixup",""
"atherose2200ethernet","AtherosE2200","AtherosE2200Ethernet",""
"azulpatcher4600","AzulPatcher4600","AzulPatcher4600",""
"brcmpatchram","BrcmFirmwareStore","BrcmPatchRam",""
"bt4lecontiunityfixup","BT4LEContiunityFixup","BT4LEContiunityFixup",""
"clover","Clover","Clover Bootloader",""
"codeccommander","CodecCommander","CodecCommander",""
"coredisplayfixup","CoreDisplayFixup","CoreDisplayFixup","WhateverGreen","Alarm"
"cpufriend","CPUFriend","CPUFriend",""
"enablelidwake","EnableLidWake","EnableLidWake",""
"fakepciid","FakePCIID","FakePCIID",""
"fakesmc","FakeSMC","FakeSMC",""
"hibernationfixup","HibernationFixup","HibernationFixup",""
"intelgraphicsdvmtfixup","IntelGraphicsDVMTFixup","IntelGraphicsDVMTFixup","WhateverGreen","Alarm"
"intelgraphicsfixup","IntelGraphicsFixup","IntelGraphicsFixup","WhateverGreen","Alarm"
"intelmausiethernet","AppleIntelE1000","AppleIntelE1000","IntelMausiEthernet"
"intelmausiethernet","IntelMausi","IntelMausiEthernet",""
"lilu","Lilu ","Lilu",""
"lilufriend","LiluFriend","LiluFriend",""
"nightshiftunlocker","NightShiftUnlocker","NightShiftUnlocker",""
"notouchid","NoTouchID","NoTouchID",""
"nullcpupowermanagement","NullCpuPower","NullCpuPowerManagement",""
"nullethernet","NullEthernet","NullEthernet",""
"nvidiagraphicsfixup","LibValFix","NVWebDriverLibValFix","WhateverGreen","Alarm"
"nvidiagraphicsfixup","NvidiaGraphicsFixup","NvidiaGraphicsFixup","WhateverGreen","Alarm"
"realtekrtl8111","RealtekRTL8111","RealtekRTL8111",""
"shiki","Shiki","Shiki","WhateverGreen","Alarm"
"usbinjectall","USBInjectAll","USBInjectAll",""
"virtualsmc","VirtualSMC","VirtualSMC",""
"voodoohda","VoodooHDA","VoodooHDA",""
"voodooi2c","VoodooI2C (","VoodooI2C",""
"whatevergreen","WhateverGreen","WhateverGreen",""
"voodoops2","PS2Controller","VoodooPS2",""
)

#========================= Language Detection =========================#
function _languageselect()
{
    if [[ $lan2 = de* ]]; then
    export LC_ALL=de_DE
    language="de"
    elif [[ $lan2 = tr* ]]; then
    export LC_ALL=tr_TR
    language="tr"
    elif [[ $lan2 = ru* ]]; then
    export LC_ALL=ru_RU
    language="ru"
    elif [[ $lan2 = uk* ]]; then
    export LC_ALL=uk_UK
    language="uk"
    elif [[ $lan2 = es* ]]; then
    export LC_ALL=es_ES
    language="es"
    elif [[ $lan2 = pt* ]]; then
    export LC_ALL=pt_PT
    language="pt-PT"
    elif [[ $lan2 = nl* ]]; then
    export LC_ALL=nl_NL
    language="nl"
    elif [[ $lan2 = fr* ]]; then
    export LC_ALL=fr_FR
    language="fr"
    elif [[ $lan2 = it* ]]; then
    export LC_ALL=it_IT
    language="it"
    elif [[ $lan2 = fi* ]]; then
    export LC_ALL=fi_FI
    language="fi"
    elif [[ $lan2 = pl* ]]; then
    export LC_ALL=pl_PL
    language="pl"
    elif [[ $lan2 = sv* ]]; then
    export LC_ALL=sv_SV
    language="sv"
    elif [[ $lan2 = cs* ]]; then
    export LC_ALL=cs_CS
    language="cs"
    else
    export LC_ALL=en_EN
    language="en"
    fi
    if [ ! -d "$ScriptTmpPath" ]; then
        mkdir "$ScriptTmpPath"
    fi
    if [ ! -f ${ScriptTmpPath}/locale.tmp ]; then
        cat ../$language.lproj/MainMenu.strings | sed "s/\=\ \"/\=\"/g" | grep -A 10000000 BASH | tail -n +2 > ${ScriptTmpPath}/locale.tmp
    fi
    source ${ScriptTmpPath}/locale.tmp
}

################################################################
####################### Initial Function #######################
################################################################

function initial()
{
    efiscan=`../bin/./BDMESG |grep -e "SelfDevicePath" -e "Found Storage" | sed -e s/".*GPT,//g" -e "s/.*MBR,//g" -e "s/,.*//g" | xargs`
    diskscan=`diskutil info $efiscan`

    efiroot=`echo -e "$efiscan"`
    devicenode=`echo -e "$diskscan" | grep "Device Node:" | sed "s/.*://g" | xargs`
    devicenodemain=`echo -e "$devicenode" | sed "s/s[0-9]//g"`
    drivemodel=`diskutil info $devicenodemain |grep "Media Name:" | sed "s/.*://g" |xargs`
    devicelocation=`echo -e "$diskscan" | grep "Device Location:" | sed "s/.*://g" | xargs`
    removablemedia=`echo -e "$diskscan" | grep "Removable Media:" | sed "s/.*://g" | xargs`
    mounted=`echo -e "$diskscan" | grep "Mounted:" | sed "s/.*://g" | xargs`
    mountpoint=`echo -e "$diskscan" | grep "Mount Point:" | sed "s/.*://g" | xargs`
    solidstate=`echo -e "$diskscan" | grep "Solid State:" | sed "s/.*://g" | xargs`
    protocol=`echo -e "$diskscan" | grep "Protocol:" | sed "s/.*://g" | xargs`
    kuroot=`pwd | sed "s/\/Kext.*//g"`

    if [[ $mountpoint != "" ]];then
        efipath=`find "$mountpoint" -name "config.plist" |sed -e "s/\.//g" -e "s/con.*//g" |grep -v "Trashes"`
        if [[ $efipath = "" ]]; then
            efipath=`find "$mountpoint" -name "Default.plist" |sed -e "s/\.//g" -e "s/con.*//g" |grep -v "Trashes"`
        fi
    fi

    _helpDefaultWrite "EFI Path" "$efipath"
    _helpDefaultWrite "EFI Root" "$efiroot"
    _helpDefaultWrite "KU Root" "$kuroot"
    _helpDefaultWrite "Device Node" "$devicenode"
    _helpDefaultWrite "Device Location" "$devicelocation"
    _helpDefaultWrite "Removable Media" "$removablemedia"
    _helpDefaultWrite "Solid State" "$solidstate"
    _helpDefaultWrite "Removable Media" "$removablemedia"
    _helpDefaultWrite "Mounted" "$mounted"
    _helpDefaultWrite "Mount Point" "$mountpoint"
    _helpDefaultWrite "Drive Model" "$drivemodel"
    _helpDefaultWrite "Device Protocol" "$protocol"
}

#################################################################
####################### Mountefi Function #######################
#################################################################

function mountefi()
{
    efiroot=`_helpDefaultRead "EFI Root"`
    mounted=`_helpDefaultRead "Mounted"`

    if [[ $mounted = "Yes" ]]; then
        diskutil unmount $efiroot >/dev/null 2>&1
        if [ $? = 0 ]; then
            _helpDefaultWrite "Mounted" "No"
        fi
    else
        diskutil mount $efiroot >/dev/null 2>&1
        status="$?"
        if [ $status = "0" ]; then
            _helpDefaultWrite "Mounted" "Yes"
        fi
        if [ $status != "0" ]; then
            osascript -e 'do shell script "diskutil mount $(../bin/./BDMESG |grep -e \"SelfDevicePath\" -e \"Found Storage\" |sed -e 's/.*GPT,//' -e 's/,0x.*//')" with administrator privileges' >/dev/null 2>&1
            if [ $? = 0 ]; then
                _helpDefaultWrite "Mounted" "Yes"
            fi
        fi
        efiscan=`../bin/./BDMESG |grep -e "SelfDevicePath" -e "Found Storage" | sed -e s/".*GPT,//g" -e "s/.*MBR,//g" -e "s/,.*//g" | xargs`
        diskscan=`diskutil info $efiscan`
        mountpoint=`echo -e "$diskscan" | grep "Mount Point:" | sed "s/.*://g" | xargs`
        _helpDefaultWrite "Mount Point" "$mountpoint"

        if [[ $mountpoint != "" ]];then
            efipath=`find "$mountpoint" -name "config.plist" |sed -e "s/\.//g" -e "s/con.*//g" |grep -v "Trashes"`
            if [[ $efipath = "" ]]; then
                efipath=`find "$mountpoint" -name "Default.plist" |sed -e "s/\.//g" -e "s/con.*//g" |grep -v "Trashes"`
            fi
        fi

        _helpDefaultWrite "EFI Path" "$efipath"
    fi
    exit 0
}

###################################################################
####################### Mainscript Function #######################
###################################################################

function mainscript()
{
#========================= Script Pathes =========================#
    ScriptDownloadPath=`_helpDefaultRead "Downloadpath"`

#========================== Set Variables =========================#
    content=`../bin/./PlistBuddy -c Print "${ScriptHome}/Library/Preferences/com.slsoft.kextupdater.plist"`
    url=`echo -e "$content" | grep "Updater URL" | sed "s/.*\=\ //g" | xargs`
    lan2=`echo -e "$content" | grep "Language" | sed "s/.*\=\ //g" | xargs`
    username=`echo -e "$content" | grep "User Name" | sed "s/.*\=\ //g" | xargs`
    realname=`echo -e "$content" | grep "Full Name" | sed "s/.*\=\ //g" | xargs`
    efiroot=`echo -e "$content" | grep "EFI Root" | sed "s/.*\=\ //g" | xargs`
    kexte=`echo -e "$content" | grep "Choice" | sed "s/.*\=\ //g" | xargs`
    webdr2=`echo -e "$content" | grep "Webdriver Build" | sed "s/.*\ \-//g" | xargs`
    singlekext=`echo -e "$content" | grep "Load Single Kext" | sed "s/.*\=\ //g" | xargs`
    nightly=`echo -e "$content" | grep "Clover Nightly" | sed "s/.*\=\ //g" | xargs`
    osuser=`echo $realname`
    hour=`date "+%H"`
    notifications=`echo -e "$content" | grep "Notifications" | sed "s/.*\=\ //g" | xargs`
    notificationsseconds=`echo -e "$content" | grep "NotificationSeconds" | sed "s/.*\=\ //g" | xargs`

    _languageselect

    os=`_helpDefaultRead "OSVersion"`
    overview=`curl -sS -A "curl/osx - $os - $lan2" https://$url/overview.html`

#========================= Check APFS =========================#
function _apfscheck()
{
    apfscheck=`../bin/./BDMESG |grep "APFS driver"` # Checks if apfs.efi is loaded

    if [ -d /Volumes/ESP ]; then
        efipath="ESP"
    else
        efipath="EFI"
    fi

    if [[ $apfscheck != "" ]]; then
        if [ ! -d /Volumes/$efipath ]; then
            efi="off"
            diskutil mount $efiroot >/dev/null 2>&1
            apfstrash=`find /Volumes/$efipath/.Trashes -name "apfs.efi"`
            if [ -f $apfstrash ]; then # Deletes apfs.efi from Trashcan if its there
                rm $apfstrash >/dev/null 2>&1
            fi
        fi
        if [ -d /Volumes/$efipath ]; then
            apfstrash=`find /Volumes/$efipath/.Trashes -name "apfs.efi"`
            if [ -f $apfstrash ]; then # Deletes apfs.efi from Trashcan if its there
                rm $apfstrash >/dev/null 2>&1
            fi
                apfspath=`find /Volumes/$efipath -name "apfs.efi" |head -n 1`
            if [[ $apfspath = *apfs.efi* ]]; then # Check if apfs.efi is in place
                apfs=`cat "$apfspath" |xxd | grep -A 2 APFS | head -n 2 | tail -n 1 | sed -e "s/.*\ \ //g" -e "s/\.\.\..*//g" -e "s/\///g"`
            fi
        fi
        if [[ $efi = "off" ]]; then #If the EFI wasn´t mounted before executing the Kext Updater it will be unmounted "politely"
            if [[ $efipath = "EFI" ]]; then
                diskutil umount $efiroot >/dev/null 2>&1
            fi
        fi
    fi
}

#========================= Check for nVidia Webdriver =========================#
checkweb=`echo -e "$kextstats" |grep web.GeForceWeb`
if [[ $checkweb != "" ]]; then
    locweb=`cat /Library/Extensions/NVDAStartupWeb.kext/Contents/Info.plist |grep NVDAStartupWeb\ | sed -e "s/.*Web\ //g" -e "s/<\/.*//g" |cut -c 10-99`
    kextstats=`echo -e "$kextstats" "\nNVDAStartupWeb ($locweb)"`
fi

#========================= Add Non-Kext Values =========================#
bdmesg=`../bin/./BDMESG |grep "Clover revision" |sed -e "s/.*revision:\ /Clover\ (/g" -e "s/\ on.*/)/g"`
kextstats=`echo -e "$kextstats" "\n$bdmesg"`
kextstats=`echo -e "$kextstats" |sed "s/d0)/)/g"`
kextstats=`echo -e "$kextstats" "\nAPFS ($apfs)"`

#========================= Excluded Kexts =========================#

function _excludedkexts()
{
    kextstatsori=`kextstat | grep -v com.apple`
    bdmesg=`../bin/./BDMESG |grep "Clover revision" |sed -e "s/.*revision:\ /Clover\ (/g" -e "s/\ on.*/)/g"`
    kextstatsori=`echo -e "$kextstatsori" "\n$bdmesg"`
    kextstatsori=`echo -e "$kextstatsori" |sed "s/d0)/)/g"`

    kext="ACPIBatteryManager"
    check=`echo "$content" | grep ex-$kext | sed "s/.*=\ //g"`
        if [[ $check = "true" ]]; then
            kextstats=`echo "$kextstatsori" | grep -v $kext`
            else
            kextstats=`echo "$kextstatsori"`
        fi

    array=($allkextsupper)
    for i in "${array[@]}"; do
    check=`echo "$content" | grep -w ex-$i | sed "s/.*=\ //g"`
        if [[ $check = "true" ]]; then
            kextstats=`echo "$kextstats" | grep -v $i`
            else
            kextstats=`echo "$kextstats"`
        fi
    done
}

#========================= Get loaded Kexts =========================#
_excludedkexts

#========================= Ozmosis Warning =========================#
function _ozmosis()
{
    ../bin/./BDMESG | grep "Ozmosis" > /dev/null
    if [[ $? = "0" ]]; then
        echo $ozmosis
        echo " "
    fi
}

#========================= Output Headline =========================#
function _printHeader()
{
    if [ $hour -lt 12 ]; then
        echo $greet1
    elif [ $hour -lt 18 ]; then
        echo $greet2
    else
        echo $greet3
    fi
        echo " "
}

#========================= KextUpdate =========================#
function _kextUpdate()
{
    for kextList in "${kextArray[@]}"
        do
            IFS=","
            data=($kextList)
            name=${data[0]}
            lecho=`echo -e "$kextstats" |grep ${data[1]} | sed -e "s/.*(//g" -e "s/).*//g"`
            local=`echo $lecho | sed -e "s/\.//g"`
            if ! [[ $local = "" ]]; then
                echo "$checkver ${data[2]} ..."
                if ! [[ -z ${data[3]} ]] ; then # veralteter Kext
                    _obsoleteKext
                fi
                if ! [[ -z ${data[4]} ]] ; then # Needs Lilu Kext
                    echo ""
                    echo "$alarm"
                    echo ""
                fi
                remote=`echo -e "$overview" |grep -w $name |sed -e "s/.*-//g" -e "s/+.*//g"`
                recho=`echo -e "$overview" |grep -w $name |sed "s/.*+//g"`
                if [ -f ${ScriptDownloadPath}/${name}/.version.htm ]; then
                    dupe=`cat ${ScriptDownloadPath}/${name}/.version.htm`
                    if [[ $dupe = $remote ]]; then
                        _dupeKext
                    fi
                else
                    returnVALUE=$(expr $local '<' $remote)
                    if [[ $returnVALUE == "1" ]]; then
                        mkdir -p ${ScriptDownloadPath} ${ScriptDownloadPath}/${name}
                        _toUpdate
                        curl -sS -o ${ScriptTmpPath}/${name}.zip https://$url/${name}/${name}.zip
                        curl -sS -o ${ScriptDownloadPath}/$name/.version.htm https://$url/${name}/version.htm
                        unzip -o -q ${ScriptTmpPath}/${name}.zip -d ${ScriptDownloadPath}/${name}
                        rm ${ScriptTmpPath}/${name}.zip 2> /dev/null
                        echo "${data[2]}" >> ${ScriptTmpPath}/kextloaded
                    else
                        _noUpdate
                    fi
                fi
            fi
        done
        _helpDefaultWrite "Clover Nightly" ""
}

#============================== KextLoad ==============================#
function _kextLoader()
{
    for kextLoadList in "${kextLoadArray[@]}"
        do
            IFS=","
            data=($kextLoadList)
            name=${data[0]}
            if [[ $nightly = "9" ]]; then
                name="clovernightly"
            fi
            mkdir -p ${ScriptDownloadPath} ${ScriptDownloadPath}/${name}
            _toUpdateLoad
            curl -sS -o ${ScriptTmpPath}/${name}.zip https://$url/${name}/${name}.zip
            curl -sS -o ${ScriptDownloadPath}/$name/.version.htm https://$url/${name}/version.htm
            unzip -o -q ${ScriptTmpPath}/${name}.zip -d ${ScriptDownloadPath}/${name}
            rm ${ScriptTmpPath}/${name}.zip 2> /dev/null
    done
}

#========================= KextReport =========================#
function _kextReport()
{
    kextstats=`kextstat | grep -v com.apple`
    date > "$ScriptReportPath"/Kext-Report.txt
    echo "" >> "$ScriptReportPath"/Kext-Report.txt
    sw_vers |tail -n2 >> "$ScriptReportPath"/Kext-Report.txt
    echo "" >> "$ScriptReportPath"/Kext-Report.txt
    echo "$kextloaded" >> "$ScriptReportPath"/Kext-Report.txt
    echo "────────────────" >> "$ScriptReportPath"/Kext-Report.txt
    for kextList in "${kextArray[@]}"
        do
            IFS=","
            data=($kextList)
            name=${data[0]}
            lecho=`echo -e "$kextstats" |grep ${data[1]} | sed -e "s/.*(//g" -e "s/).*//g"`
            local=`echo $lecho | sed -e "s/\.//g"`
            if ! [[ $local = "" ]]; then
                echo "${data[2]}" "($lecho)" >> "$ScriptReportPath"/Kext-Report.txt
            fi
    done
    echo "────────────────" >> "$ScriptReportPath"/Kext-Report.txt
    echo "" >> "$ScriptReportPath"/Kext-Report.txt
    kextstat | grep AppleHDA\ \( > /dev/null
    if [ $? = 0 ]; then
        echo "$hdaloaded" >> "$ScriptReportPath"/Kext-Report.txt
    else
        echo "$hdanotloaded" >> "$ScriptReportPath"/Kext-Report.txt
    fi
    echo "" >> "$ScriptReportPath"/Kext-Report.txt
    echo "$reportpath": >> "$ScriptReportPath"/Kext-Report.txt
    echo "$ScriptReportPath"/Kext-Report.txt >> "$ScriptReportPath"/Kext-Report.txt

    cat "$ScriptReportPath"/Kext-Report.txt
}

#========================= Helpfunction Update =========================
function _toUpdate()
{
    _PRINT_MSG "$upd1\n
    $upd2 $lecho
    $upd3 $recho\n\n
    $loading\n────────────────\n"
}

function _toUpdateLoad()
{
    namec=`echo $name | awk '{print toupper(substr($0,0,1))tolower(substr($0,2))}'`
    _PRINT_MSG "$namec $dloading\n────────────────"
}

#========================= Helpfunction no Duplicates =========================#
function _dupeKext()
{
    _PRINT_MSG "$dupekext\n────────────────"
}

#========================= Helpfunction no Kext =========================#
function _noUpdate()
{
    _PRINT_MSG "$upd4 ($lecho)\n────────────────"
}

#========================= Helpfunction obsolete Kext =========================#
function _obsoleteKext()
{
    _PRINT_MSG "$obsolete1 ${data[2]} $obsolete2 ${data[3]} $obsolete3"
}
#========================= Helpfunction Message =========================#
function _PRINT_MSG()
{
    local message=$1
    printf "${message}\n"
}

#============================== Webdriver ==============================#
function _nvwebdriver()
{
    mkdir -p "$ScriptDownloadPath/nVidia Webdriver"
    echo Build $webdr2 $webdrload
    curl -sS -o "$ScriptDownloadPath/nVidia Webdriver/$webdr2.pkg" https://$url/nvwebdriver/$webdr2.pkg
    echo " "
    echo $webdrloaded
    exit 0
}

#============================== Cleanup Files ==============================#
function _cleanup()
{
    find $ScriptDownloadPath/ -name *.dSYM -exec rm -r {} \; >/dev/null 2>&1
    find $ScriptDownloadPath/ -name __MACOSX -exec rm -r {} \; >/dev/null 2>&1
    if [ -f ${ScriptTmpPath}/kextloaded ]; then
        rm ${ScriptTmpPath}/kextloaded
    fi
}

#============================== Kext Updater Last Run ==============================#
function _lastcheck()
{
    lastcheckfunc=$( _helpDefaultRead "Last Check" )
    if [[ $lastcheckfunc != "Never" ]]; then
        echo $lastcheck
        echo $lastcheckfunc
        echo " "
    fi
}

#============================== Main Function ==============================#
function _main()
{
    if [[ $1 == kextUpdate ]]; then
        _printHeader
        _lastcheck
    fi
    if [[ $1 == kextUpdate ]]; then
        _kextUpdate
        if [[ $notifications = "true" ]]; then
            if [ -f ${ScriptTmpPath}/kextloaded ]; then
                amount=`cat "$ScriptTmpPath/kextloaded" | wc -l | xargs`
            else
                amount="0"
            fi
            ../bin/./alerter -message "$notify1 $amount" -title "Kext Updater" -timeout $notificationsseconds & > /dev/null
        fi
    elif [[ $1 == kextLoader ]]; then
        _kextLoader
        if [[ $notifications = "true" ]]; then
            ../bin/./alerter -message "$notify2" -title "Kext Updater" -timeout $notificationsseconds & > /dev/null
        fi
    elif [[ $1 == kextReport ]]; then
        _kextReport
        if [[ $notifications = "true" ]]; then
            ../bin/./alerter -message "$notify3" -title "Kext Updater" -timeout $notificationsseconds & > /dev/null
        fi
    fi
    _ozmosis

    echo ""
    echo "$webdrloaded"
    _cleanup
}

#============================== Choice ==============================#
if [ $kexte = "1" ]; then
    _main "kextUpdate"
    exit 0
fi

if [ $kexte = "2" ]; then
    kextLoadArray=("fakesmc" "usbinjectall" "voodoops2")
    recho="Kexte"
    _main "kextLoader"
    exit 0
fi

if [ $kexte = "3" ]; then
    kextLoadArray=("applealc" "lilu" "codeccommander")
    recho="Kexte"
    _main "kextLoader"
    exit 0
fi

if [ $kexte = "4" ]; then
    kextLoadArray=("whatevergreen" "lilu")
    recho="Kexte"
    _main "kextLoader"
    exit 0
fi

if [ $kexte = "5" ]; then
    kextLoadArray=("atherose2200ethernet" "intelmausiethernet" "realtekrtl8111")
    recho="Kexte"
    _main "kextLoader"
    exit 0
fi

if [ $kexte = "m" ]; then
    if [ ! -f "$ScriptTmpPath"/massdownload ]; then
        echo "$nokextselected"
        exit 0
    fi
    source /tmp/kextupdater/array
    recho="Kexte"
    _main "kextLoader"
    exit 0
fi

if [ $kexte = "8" ]; then
    kextLoadArray=("clover")
    recho="Kexte"
    _main "kextLoader"
    exit 0
fi

if [ $kexte = "r" ]; then
    _main "kextReport"
    exit 0
fi

if [ $kexte = "b" ]; then
    if [[ $webdr2 = "" ]]; then
        echo "$nowebselected"
    else
        _nvwebdriver
    fi
    exit 0
fi

if [ $kexte = "d" ]; then
    if [[ $singlekext = "" ]]; then
        echo "$nokextselected"
    else
        kextchoice=`_helpDefaultRead "Load Single Kext"`
        kextLoadArray=("$kextchoice")
        recho="Kexte"
        _main "kextLoader"
    fi
    exit 0
fi

#=========================== START ===========================#
_main "kextUpdate"
exit 0
#=============================================================#
}

###################################################################
######################## Rebuild Kext-Cache #######################
###################################################################

function rebuildcache()
{
    content=`../bin/./PlistBuddy -c Print "${ScriptHome}/Library/Preferences/com.slsoft.kextupdater.plist"`
    lan2=`echo -e "$content" | grep "Language" | sed "s/.*\=\ //g" | xargs`

    _languageselect

    echo "$rebuildcache"
    echo ""
    osascript -e 'do shell script "sudo chmod -R 755 /System/Library/Extensions/*; sudo chown -R root:wheel /System/Library/Extensions/*; sudo touch /System/Library/Extensions; sudo kextcache -i /; sudo touch /System/Library/Extensions; sudo kextcache -u / -v 6" with administrator privileges' >/dev/null 2>&1
    if [ $? = 0 ]; then
        echo "$webdrloaded"
        else
        echo "$error"
    fi
}

###################################################################
############################# Exit App ############################
###################################################################

function exitapp()
{
    ScriptDownloadPath=`_helpDefaultRead "Downloadpath"`
    find "$ScriptDownloadPath" -name ".version.htm" -exec rm {} \;
}

###################################################################
################ Reset Preferences and restart App ################
###################################################################

function resetprefs()
{
    apppath=`ps ax |grep Kext\ Updater.app | head -n1 | sed -e "s/.*\ \//\//g" -e "s/\/Con.*//g"`
    echo "rm ~/Library/Preferences/com.slsoft.kextupdater.plist" > /tmp/kurestarter
    echo "for KILLPID in \`ps ax | grep 'Kext Updater' | awk ' { print \$1;}'\`; do" >> /tmp/kurestarter
    echo "kill -9 \$KILLPID;" >> /tmp/kurestarter
    echo "done" >> /tmp/kurestarter
    echo "sleep 1" >> /tmp/kurestarter
    echo open \"$apppath\" >> /tmp/kurestarter
    echo "rm /tmp/kurestarter" >> /tmp/kurestarter
    bash /tmp/kurestarter
}

###################################################################
####################### Kext Mass Download ########################
###################################################################

function massdownload()
{
    content=`../bin/./PlistBuddy -c Print "${ScriptHome}/Library/Preferences/com.slsoft.kextupdater.plist"`
    lan2=`echo -e "$content" | grep "Language" | sed "s/.*\=\ //g" | xargs`

    _languageselect

    if [ -f "$ScriptTmpPath"/massdownload ]; then
        rm "$ScriptTmpPath"/massdownload
    fi

    array=($allkextslower)
    for i in "${array[@]}"; do
        check=`echo -e "$content" | grep -w "$i" | sed "s/.*\=\ //g" | xargs`
        if [[ $check = "true" ]]; then
            echo "$i" >> "$ScriptTmpPath"/massdownload
        fi
        if [[ $check != "" ]]; then
            _helpDefaultDelete "$i"
        fi
    done

    cat "$ScriptTmpPath"/massdownload | sed -e 's/^/"/' -e 's/$/"\ /' | tr -d '\n' > "$ScriptTmpPath"/massdownload2
    kexts=`cat "$ScriptTmpPath"/massdownload2`
    echo "kextLoadArray=($kexts)" | sed "s/\ )/)/g" > "$ScriptTmpPath"/array

    if [ -f "$ScriptTmpPath"/massdownload ]; then
        echo "$massdlready"
        else
        echo "$nokextselected"
    fi
    exit 0
}

###################################################################
##################### Reset excluded Kexts ########################
###################################################################
function excludereset()
{
    array=($allkextsupper)
    for i in "${array[@]}"; do
        _helpDefaultDelete "ex-$i" 2> /dev/null
    done
    exit 0
}

###################################################################
##################### Kext Updater Daemon #########################
###################################################################
function kudaemon()
{
    content=`../bin/./PlistBuddy -c Print "${ScriptHome}/Library/Preferences/com.slsoft.kextupdater.plist"`
    lan2=`echo -e "$content" | grep "Language" | sed "s/.*\=\ //g" | xargs`
    url=`echo -e "$content" | grep "Updater URL" | sed "s/.*\=\ //g" | xargs`

    _languageselect

    _excludedkexts

    echo "$kextstats" | tr '[:upper:]' '[:lower:]' > "$ScriptTmpPath"/daemon_kextstat

    curl -sS -A "KU Daemon" -o "$ScriptTmpPath"/daemon_overview https://$url/overview.html

    while IFS='' read -r line || [[ -n "$line" ]]; do
        kext=`echo "$line" |sed "s/-.*//g"`
        kstat=`grep -w "$kext" "$ScriptTmpPath"/daemon_kextstat | sed -e "s/.*(//g" -e "s/).*//g" -e "s/\.//g" -e "s/d0//g"`
        kover=`grep -w "$kext" "$ScriptTmpPath"/daemon_overview | sed -e "s/.*-//g" -e "s/+.*//g"`

        if [[ $kstat != "" ]]; then
            if [[ "$kover" -gt "$kstat" ]]; then
                touch "$ScriptTmpPath"/daemon_notify
            fi
        fi
    done < "$ScriptTmpPath"/daemon_overview

    if [ -f "$ScriptTmpPath"/daemon_notify ]; then
        kuroot=`_helpDefaultRead "KU Root"`
        ANSWER="$(../bin/./alerter -message "$daemonnotify" -title "Kext Updater Daemon" -actions "$openkextupdater" -closeLabel "$daemonnotifyclose" -appIcon https://kextupdater.slsoft.de/kextupdater/appicon.png)"
        case $ANSWER in
        "$openkextupdater") open "$kuroot"/Kext\ Updater.app ;;
        esac
        rm "$ScriptTmpPath"/daemon_*
    fi
    exit 0
}

###################################################################
################ Kext Updater Daemon LoginItem ####################
###################################################################
function loginitem_on()
{
    kuroot=`_helpDefaultRead "KU Root" | sed -e "s/$/\/Kext\ Updater.app\/Contents\/Resources\/bin\/KUDaemon.app/"`
    osascript -e 'tell application "System Events" to delete login item "KUDaemon"' > /dev/null
    osascript -e 'tell application "System Events" to make login item at end with properties {path:"'"$kuroot"'", hidden:false}' > /dev/null
}

function loginitem_off()
{
    osascript -e 'tell application "System Events" to delete login item "KUDaemon"' > /dev/null
}

$1
